/* 
* ################################################################
* 
* ProActive: The Java(TM) library for Parallel, Distributed, 
*            Concurrent computing with Security and Mobility
* 
* Copyright (C) 1997-2002 INRIA/University of Nice-Sophia Antipolis
* Contact: proactive-support@inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               The ProActive Team
*                        http://www.inria.fr/oasis/ProActive/contacts.html
*  Contributor(s): 
* 
* ################################################################
*/ 
package org.objectweb.proactive.core.group.spmd;

import java.lang.reflect.InvocationTargetException;

import org.objectweb.proactive.ProActive;
import org.objectweb.proactive.core.body.AbstractBody;
import org.objectweb.proactive.core.group.MethodCallControlForGroup;
import org.objectweb.proactive.core.mop.MethodCallExecutionFailedException;

/**
 * @author Laurent Baduel
 */
public class MethodCallSetSPMDGroup extends MethodCallControlForGroup {

	/**
	 * Builds a method call to set the SPMD group   
	 * @param smpgGroup - the SPMD group to set
	 */
	public MethodCallSetSPMDGroup(Object smpgGroup) {
		Object[] objTab = new Object[1];
		objTab[0] = smpgGroup;
		this.setEffectiveArguments(objTab);
	}

	/**
	 * Returns the name of the call
	 * @return the String "MethodCallSetSPMDGroup";
	 */
	public String getName() {
		return "MethodCallSetSPMDGroup";
	}

	/**
	 * This call have one parameter.
	 * @return 1
	 * @see org.objectweb.proactive.core.mop.MethodCall#getNumberOfParameter()
	 */
	public int getNumberOfParameter() {
		return 1;
	}

	/**
	 * Executes the call.
	 * @see org.objectweb.proactive.core.mop.MethodCall#execute(java.lang.Object)
	 */
	public Object execute(Object targetObject)
		throws InvocationTargetException, MethodCallExecutionFailedException {
		
		AbstractBody body = (AbstractBody) ProActive.getBodyOnThis();
		body.setSPMDGroup(this.getParameter(0));
		return null;
	}

}
