/* 
* ################################################################
* 
* ProActive: The Java(TM) library for Parallel, Distributed, 
*            Concurrent computing with Security and Mobility
* 
* Copyright (C) 1997-2002 INRIA/University of Nice-Sophia Antipolis
* Contact: proactive-support@inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               The ProActive Team
*                        http://www.inria.fr/oasis/ProActive/contacts.html
*  Contributor(s): 
* 
* ################################################################
*/ 
package org.objectweb.proactive.core.mop;

public abstract class MOPRuntimeException extends RuntimeException {

  protected Throwable detail;

  /**
  * Constructs a <code>ProActiveRuntimeException</code> with no specified
  * detail message.
  */
  public MOPRuntimeException() {}

  /**
  * Constructs a <code>ProActiveRuntimeException</code> with the specified detail message.
  * @param s the detail message
  */
  public MOPRuntimeException(String s) {
    super(s);
  }

  /**
  * Constructs a <code>ProActiveRuntimeException</code> with the specified
  * detail message and nested exception.
  *
  * @param s the detail message
  * @param ex the nested exception
  */
  public MOPRuntimeException(String s, Throwable ex) {
    super(s);
    detail = ex;
  }

  /**
  * Constructs a <code>ProActiveException</code> with the specified
  * detail message and nested exception.
  *
  * @param ex the nested exception
  */
  public MOPRuntimeException(Throwable ex) {
    super();
    detail = ex;
  }

  /**
  * Returns the detail message, including the message from the nested
  * exception if there is one.
  */
  public String getMessage() {
    if (detail == null)
      return super.getMessage();
    else {
      if (super.getMessage() == null)
        return detail.getMessage();
      else return super.getMessage() + "; nested exception is: \n" + detail.toString();
    }
  }


  public Throwable getTargetException() {
    return detail;
  }


  /**
  * Prints the composite message and the embedded stack trace to
  * the specified stream <code>ps</code>.
  * @param ps the print stream
  */
  public void printStackTrace(java.io.PrintStream ps) {
    if (detail == null) {
      super.printStackTrace(ps);
    } else {
      synchronized(ps) {
        ps.println(getMessage());
        detail.printStackTrace(ps);
      }
    }
  }

  /**
  * Prints the composite message to <code>System.err</code>.
  */
  public void printStackTrace() {
    printStackTrace(System.err);
  }

  /**
  * Prints the composite message and the embedded stack trace to
  * the specified print writer <code>pw</code>.
  * @param pw the print writer
  */
  public void printStackTrace(java.io.PrintWriter pw) {
    if (detail == null) {
      super.printStackTrace(pw);
    } else {
      synchronized(pw) {
        pw.println(getMessage());
        detail.printStackTrace(pw);
      }
    }
  }
  
}

