/* 
* ################################################################
* 
* ProActive: The Java(TM) library for Parallel, Distributed, 
*            Concurrent computing with Security and Mobility
* 
* Copyright (C) 1997-2002 INRIA/University of Nice-Sophia Antipolis
* Contact: proactive-support@inria.fr
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or any later version.
*  
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
* USA
*  
*  Initial developer(s):               The ProActive Team
*                        http://www.inria.fr/oasis/ProActive/contacts.html
*  Contributor(s): 
* 
* ################################################################
*/ 
package org.objectweb.proactive.ic2d.util;

/**
 * 
 */
public class RunnableProcessor {

  private static RunnableProcessor singletonInstance = new RunnableProcessor();

  private IC2DMessageLogger logger;
  
  //
  // -- CONSTRUCTORS -----------------------------------------------
  //

  private RunnableProcessor() {
  }
  
  //
  // -- PUBLIC METHODS -----------------------------------------------
  //
  
  public static RunnableProcessor getInstance() {
    return singletonInstance;
  }

  public void processRunnable(Runnable task, IC2DMessageLogger logger) {
    processRunnable("Unamed Task", task, logger);
  }
  
  public void processRunnable(String taskName, Runnable task, IC2DMessageLogger logger) {
    RunnerThread t = new RunnerThread(taskName, task);
    this.logger = logger;
    t.start();
    //System.out.println("Started runnable : "+taskName);
  }
  
  //
  // -- PRIVATE METHODS -----------------------------------------------
  //

  //
  // -- INNER CLASSES -----------------------------------------------
  //
  
  private class RunnerThread extends Thread {
    
    private Runnable task;
    
    public RunnerThread(String name, Runnable task) {
      super(name);
      this.task = task;
    }
    
    public void run() {
      try {
        task.run();
      } catch (Exception e) {
        logger.log("The task "+getName()+" failed to execute to term", e);
      }
    }
    
  }

}
