/* Native methods for ibis.impl.messagePassing.ByteOutputStream
 */

#include <string.h>
#include <jni.h>

#include <pan_sys.h>
#include <pan_align.h>
#include <pan_util.h>
#include <pan_time.h>

#include "ibmp.h"
#include "ibmp_poll.h"
#include "ibmp_inttypes.h"

#include "ibis_impl_messagePassing_ByteOutputStream.h"
#include "ibis_impl_messagePassing_SendPort.h"

#include "ibp.h"
#include "ibp_mp.h"
#include "ibmp_byte_input_stream.h"
#include "ibmp_byte_output_stream.h"

#define JASON	0	/* 0 */

#if JASON
static struct pan_time now;
static struct pan_time start;
#endif

static int	ibmp_send_sync = 0;


static int	ibmp_byte_stream_port;
static int	ibmp_byte_stream_proto_size;
int	ibmp_byte_stream_proto_start;

static int	ibmp_byte_output_stream_alive = 0;

#ifdef IBP_VERBOSE
static int	sent_data = 0;
#endif

#define DISABLE_SENDER_INTERRUPTS	1

#if DISABLE_SENDER_INTERRUPTS
static int	intr_enable = 0;
static int	intr_disable = 0;
#endif
static int	send_frag = 0;
static int	send_first_frag = 0;
static int	send_last_frag = 0;
static int	send_msg = 0;
static int	send_frag_skip = 0;


static jclass cls_PandaByteOutputStream;

static jfieldID fld_nativeIOVec;
static jfieldID fld_waitingInPoll;
static jfieldID fld_outstandingFrags;
static jfieldID fld_makeCopy;
static jfieldID fld_msgCount;

static jmethodID md_finished_upcall;

typedef void (*release_func_t)(JNIEnv *env, void *array, void *data, jint mode);

typedef enum jprim_type {
    jprim_Boolean,
    jprim_Byte,
    jprim_Char,
    jprim_Short,
    jprim_Int,
    jprim_Long,
    jprim_Float,
    jprim_Double,
    jprim_n_types
} jprim_type_t;

typedef struct RELEASE {
    void	       *array;
    void	       *buf;
    jprim_type_t	type;
} release_t, *release_p;


typedef enum MSG_STATE {
    MSG_STATE_UNTOUCHED,
    MSG_STATE_ACCUMULATING,
    MSG_STATE_SYNC_SEND,
    MSG_STATE_ASYNC_SEND,
    MSG_STATE_CLEARING,
    MSG_STATE_N
} msg_state_t, *msg_state_p;


typedef struct IBMP_MSG ibmp_msg_t, *ibmp_msg_p;

struct IBMP_MSG {
    jobject		byte_output_stream;
    pan_iovec_p		iov;
    release_p		release;
    int			iov_len;
    int			iov_alloc_len;
    int			split_count;
    void              **proto;
    int			outstanding_send;
    jboolean		outstanding_final;
    int			copy;
    int			firstFrag;
    ibmp_msg_p		next;
    char		*buf;
    int			buf_alloc_len;
    int			buf_len;
    msg_state_t		state;
};

#define IOV_CHUNK	16
#define BUF_CHUNK	4096
#define COPY_THRESHOLD	64

static ibmp_msg_p	ibmp_msg_freelist;
static ibmp_msg_p	ibmp_sent_msg_q;


#ifndef NDEBUG

static int		freelist_size = 0;

static void
ibmp_msg_freelist_verify(JNIEnv *env, int line)
{
    ibmp_msg_p	scan;
    int		size = 0;

    for (scan = ibmp_msg_freelist; scan != NULL; scan = scan->next) {
	assert(scan->outstanding_send == 0);
	if (scan->state != MSG_STATE_UNTOUCHED) {
	    IBP_VPRINTF(1, env, ("scan %p ->state = %d\n", scan, scan->state));
	}
	assert(scan->state == MSG_STATE_UNTOUCHED);
	size++;
    }
    if (size != freelist_size) {
	fprintf(stderr, "Free list corrupt: current size %d should be %d call from %s.%d\n", size, freelist_size, __FILE__, line);
    }
    assert(size == freelist_size);
}

#else
#define ibmp_msg_freelist_verify(env, line)
#endif


static void
splitter_increase(ibmp_msg_p msg, int splitTotal)
{
    int		i;
    int		n = msg->split_count;

    msg->proto = realloc(msg->proto, sizeof(msg->proto[0]) * splitTotal);
    for (i = n; i < splitTotal; i++) {
	msg->proto[i] = pan_proto_create(ibmp_byte_stream_proto_size);
    }
    msg->split_count = splitTotal;
}



static ibmp_msg_p
ibmp_msg_get(JNIEnv *env, jobject byte_output_stream)
{
    ibmp_msg_p msg = ibmp_msg_freelist;

    if (msg == NULL) {
	IBP_VPRINTF(750, env, ("Do a poll in the hopes that we can recycle a msg struct\n"));
	ibmp_poll_locked(env);
	msg = ibmp_msg_freelist;
    }

    if (msg == NULL) {
	static int live_msgs;

	msg = pan_malloc(sizeof(*msg));
	msg->split_count = 0;
	msg->proto = NULL;
	splitter_increase(msg, 1);
	msg->iov_alloc_len = IOV_CHUNK;
	msg->iov = pan_malloc(msg->iov_alloc_len * sizeof(pan_iovec_t));
	msg->release = pan_malloc(msg->iov_alloc_len * sizeof(release_t));
	msg->outstanding_send = 0;
	msg->iov_len = 0;
	msg->buf_len = 0;
	msg->buf_alloc_len = BUF_CHUNK;
	msg->buf = pan_malloc(BUF_CHUNK);
	msg->state = MSG_STATE_ACCUMULATING;

    } else {
	assert(msg->iov_len == 0);
	assert(msg->buf_len == 0);
	assert(msg->byte_output_stream == NULL);
	assert(msg->state == MSG_STATE_UNTOUCHED);
	ibmp_msg_freelist = msg->next;
#ifndef NDEBUG
	freelist_size--;
	msg->state = MSG_STATE_ACCUMULATING;
#else
	if (msg->state != MSG_STATE_ACCUMULATING) {
	    fprintf(stderr, "%s.%d: inconsistent msg state %d\n", msg->state);
	}
#endif
    }
    msg->byte_output_stream = (*env)->NewGlobalRef(env, byte_output_stream);
    assert(ibmp_equals(env, msg->byte_output_stream, byte_output_stream));
    msg->firstFrag = 1;
#ifndef NDEBUG
    msg->next = NULL;
#endif
    assert(msg->outstanding_send == 0);
    ibmp_msg_freelist_verify(env, __LINE__);

    return msg;
}


static void
ibmp_msg_put(JNIEnv *env, ibmp_msg_p msg)
{
    IBP_VPRINTF(300, env, ("Now start free msg %p iov %d\n", msg, msg->iov_len));

    ibmp_msg_freelist_verify(env, __LINE__);

    assert(msg->byte_output_stream != NULL);
    (*env)->DeleteGlobalRef(env, msg->byte_output_stream);

#ifndef NDEBUG
    msg->byte_output_stream = NULL;
    assert(msg->outstanding_send == 0);
    assert(msg->next == NULL);
    assert(msg->state == MSG_STATE_ACCUMULATING);
    msg->state = MSG_STATE_UNTOUCHED;
    freelist_size++;
#endif

    msg->next = ibmp_msg_freelist;
    ibmp_msg_freelist = msg;
    ibmp_msg_freelist_verify(env, __LINE__);
    IBP_VPRINTF(300, env, ("Now finished free msg %p iov %d\n", msg, msg->iov_len));
}


static void
ibmp_msg_enq(ibmp_msg_p msg)
{
    msg->next = ibmp_sent_msg_q;
    ibmp_sent_msg_q = msg;
}


static void
ibmp_msg_release_iov(JNIEnv *env, ibmp_msg_p msg)
{
    int		i;

    if (msg->state == MSG_STATE_ACCUMULATING) {
	/* Lazy call. Our work has already been done. */
	return;
    }

    assert(msg->state == MSG_STATE_ASYNC_SEND ||
	    msg->state == MSG_STATE_SYNC_SEND);
#ifndef NDEBUG
    msg->state = MSG_STATE_CLEARING;
#endif
    if (msg->copy) {
	ibmp_lock_check_owned(env);
	for (i = 0; i < msg->iov_len; i++) {
	    IBP_VPRINTF(800, NULL, ("Now free msg %p iov %p size %d\n",
			msg, msg->iov[i].data, msg->iov[i].len));
	    pan_free(msg->iov[i].data);
	}
    } else {
	for (i = 0; i < msg->iov_len; i++) {
	    IBP_VPRINTF(800, NULL, ("%s: Now release msg %p iov %p size %d release type %d array %p buf %p\n",
			ibmp_currentThread(env),
			msg, msg->iov[i].data, msg->iov[i].len,
			msg->release[i].type, msg->release[i].array,
			msg->release[i].buf));
	    if (msg->release[i].type != jprim_n_types) {
		/* 'type' used to be a function pointer that indicates
		 * Release ## JType ## ArrayElements; but it seems that
		 * the Java and native array types must be correctly
		 * specified -- at least for Borland C++Builder.
		 * So now we use a switch to release them and cast the
		 * arrays back to their correct types. */
		switch (msg->release[i].type) {

#define RELEASE_ARRAY(JType, jtype) \
		    case jprim_ ## JType: \
			(*env)->Release ## JType ## ArrayElements(env, \
					(jtype ## Array)msg->release[i].array, \
					(jtype *)msg->release[i].buf, \
					JNI_ABORT); \
			break;

		    RELEASE_ARRAY(Boolean, jboolean)
		    RELEASE_ARRAY(Byte, jbyte)
		    RELEASE_ARRAY(Char, jchar)
		    RELEASE_ARRAY(Short, jshort)
		    RELEASE_ARRAY(Int, jint)
		    RELEASE_ARRAY(Long, jlong)
		    RELEASE_ARRAY(Float, jfloat)
		    RELEASE_ARRAY(Double, jdouble)

#undef RELEASE_ARRAY
		    default:
			break;
		}
		IBP_VPRINTF(300, env, ("Now delete global ref %p\n", msg->release[i].array));
		(*env)->DeleteGlobalRef(env, msg->release[i].array);
#ifndef NDEBUG
		msg->release[i].array = NULL;
#endif
		IBP_VPRINTF(755, env, ("Now deleted global ref %p\n", msg->release[i].array));
	    }
	}
    }
    msg->iov_len = 0;
    msg->buf_len = 0;

    msg->state = MSG_STATE_ACCUMULATING;
}


static void
handle_finished_send(JNIEnv *env, ibmp_msg_p msg)
{
#if JASON
   struct pan_time tt1;
   struct pan_time tt2;
#endif
    int handle;
    jboolean waitingInPoll;
    jint outstandingFrags;

    IBP_VPRINTF(300, env, ("Do a finished upcall msg %p obj %p firstFrag %s\n",
		msg, msg->byte_output_stream, msg->firstFrag ? "yes" : "no"));

#if JASON
    pan_time_get(&now);
    pan_time_sub(&now, &start);
    printf("%2d: %lf: delayed clear\n", ibp_me, pan_time_t2d(&now));
    pan_time_get(&tt1);
#endif

#ifndef NDEBUG
    assert(msg->state == MSG_STATE_ASYNC_SEND);
#endif

    // IBP_VPRINTF(300, env, ("Here...\n"));

    ibmp_msg_release_iov(env, msg);

    assert(msg->state == MSG_STATE_ACCUMULATING);

    /* We'd better be sure we own the PandaIbis lock, because these
     * (test and set)s (handle, outstandingFrags, msg->byte_output_stream)
     * otherwise are no way atomic */
    // IBP_VPRINTF(300, env, ("Here...\n"));

    waitingInPoll = (*env)->GetBooleanField(env, msg->byte_output_stream, fld_waitingInPoll);

    IBP_VPRINTF(300, env, ("Here... waitingInPoll %d\n", waitingInPoll));

    outstandingFrags = (*env)->GetIntField(env,
					   msg->byte_output_stream,
					   fld_outstandingFrags);

    if (waitingInPoll && outstandingFrags == 0) {
	IBP_VPRINTF(300, env, ("Here... outstandingFrags %d\n", outstandingFrags));
	(*env)->CallVoidMethod(env, msg->byte_output_stream, md_finished_upcall);
    } else {
	IBP_VPRINTF(300, env, ("Here... outstandingFrags %d\n", outstandingFrags));
	outstandingFrags--;
	(*env)->SetIntField(env,
			    msg->byte_output_stream,
			    fld_outstandingFrags,
			    outstandingFrags);
    }

    IBP_VPRINTF(300, env, ("Here...\n"));

    handle = (*env)->GetIntField(env, msg->byte_output_stream, fld_nativeIOVec);

    // IBP_VPRINTF(300, env, ("Here... handle %x\n", handle));
    if (handle == 0) {
	/**
	 * Ensure that there is a nativeIOVec (i.e. a data vector container)
	 * ready for a push/send for the next message
	 */
	IBP_VPRINTF(720, env, ("After Async send restore obj %p nativeIOVec to %p\n", msg->byte_output_stream, msg));
	(*env)->SetIntField(env, msg->byte_output_stream, fld_nativeIOVec, (jint)msg);
    } else if ((ibmp_msg_p)handle != msg) {
	assert(msg->next == NULL);
	ibmp_msg_put(env, msg);
	assert(msg->state == MSG_STATE_UNTOUCHED);
    }

    IBP_VPRINTF(300, env, ("Here...\n"));

#if JASON
    pan_time_get(&tt2);
    pan_time_sub(&tt2, &tt1);
    printf("upcall = %lf \n", pan_time_t2d(&tt2));
#endif
}


static int
ibmp_msg_q_poll(JNIEnv *env)
{
    ibmp_msg_p msg = ibmp_sent_msg_q;
    ibmp_msg_p prev = NULL;
    ibmp_msg_p next;
    int		done_anything = 0;

    if (0 && msg != NULL) {
	fprintf(stderr, "Queue { ");
	for (next = msg; next != NULL; next = next->next) {
	    fprintf(stderr, "%p[%d] ", msg, msg->outstanding_send);
	}
	fprintf(stderr, " } ");
    }

    while (msg != NULL) {
	next = msg->next;
	if (msg->outstanding_send == 0 /* && msg->outstanding_final */) {
	    IBP_VPRINTF(800, env, ("Handle sent upcall for msg %p\n", msg));
#ifndef NDEBUG
	    msg->next = NULL;
#endif
	    handle_finished_send(env, msg);
	    IBP_VPRINTF(800, env, ("Handled sent upcall for msg %p, firstFrag %s\n", msg, msg->firstFrag ? "yes" : "no"));
	    done_anything = 1;

	    if (prev == NULL) {
		ibmp_sent_msg_q = next;
	    } else {
		prev->next = next;
	    }
	} else {
	    prev = msg;
	}
	msg = next;
    }

    return done_anything;
}


static void
ibmp_msg_deq(ibmp_msg_p msg)
{
    ibmp_msg_p scan = ibmp_sent_msg_q;
    ibmp_msg_p prev = NULL;
    ibmp_msg_p next;

    while (scan != NULL) {
	next = scan->next;
	if (msg == scan) {
	    if (prev == NULL) {
		ibmp_sent_msg_q = next;
	    } else {
		prev->next = next;
	    }
#ifndef NDEBUG
	    msg->next = NULL;
#endif
	    break;
	}
	scan = next;
    }
    assert(scan != NULL);
}


#ifdef IBP_VERBOSE
static int ibmp_sent_msg_out;
#endif


static void
sent_upcall(void *arg)
{
    ibmp_msg_p msg = arg;
    msg->outstanding_send--;
    IBP_VPRINTF(1, NULL, ("sent upcall msg %p outstanding := %d, missing := %d\n", msg, msg->outstanding_send, --ibmp_sent_msg_out));
#if 0 && defined IBP_VERBOSE
    {
	int i;
	int j;

	fprintf(stderr, "Iovec we will clear:\n");
	for (i = 0; i < msg->iov_len; i++) {
	    fprintf(stderr, "buffer %p: ", msg->iov[i]);
	    for (j = 0; j < 24; j++) {
		fprintf(stderr, "%d ", ((int *)(msg->iov[i].data))[j]);
	    }
	    fprintf(stderr, "\n");
	}
    }
#endif
}



#ifdef IBP_VERBOSE

static void
print_byte(FILE *out, uint8_t x)
{
    fprintf(out, "0x%x ", x);
}

#define DUMP_LIMIT	128
#define DUMP_DATA(jtype, fmt, cast) \
static void dump_ ## jtype(jtype *b, int len) \
{ \
    int		i; \
    \
    if (ibmp_verbose < 300) return; \
    fprintf(stderr, "sizeof(cast) = %d\n", sizeof(cast)); \
    \
    for (i = 0; i < len; i++) { \
	if (sizeof(jtype) == sizeof(jbyte)) { \
	    print_byte(stderr, (uint8_t)b[i]); \
	} else { \
	    fprintf(stderr, "%" fmt, (cast)b[i]); \
	} \
	if (i * sizeof(jtype) >= DUMP_LIMIT) { \
	    fprintf(stderr, " ..."); \
	    break; \
	} \
    } \
    fprintf(stderr, "\n"); \
}
#else
#define DUMP_DATA(jtype, fmt, cast) \
static void dump_ ## jtype(jtype *b, int len) \
{ \
}
#endif


DUMP_DATA(jboolean, "d ", int8_t)
DUMP_DATA(jbyte, "c", int8_t)
DUMP_DATA(jchar, "d ", int16_t)
DUMP_DATA(jshort, "d ", int16_t)
DUMP_DATA(jint, "d ", int32_t)
DUMP_DATA(jlong, "lld ", int64_t)
DUMP_DATA(jfloat, "f ", float)
DUMP_DATA(jdouble, "f ", double)


static ibmp_msg_p
ibmp_msg_check(JNIEnv *env, jobject this, int nativeIOVec, int locked)
{
    ibmp_msg_p msg;

    if (nativeIOVec != 0) {
	msg = (ibmp_msg_p)nativeIOVec;
	assert(ibmp_equals(env, msg->byte_output_stream, this));
	return msg;
    }

    if (! locked) {
	ibmp_lock(env);
    }
    ibmp_msg_freelist_verify(env, __LINE__);
    msg = ibmp_msg_get(env, this);
    assert(ibmp_equals(env, msg->byte_output_stream, this));
    (*env)->SetIntField(env, this, fld_nativeIOVec, (jint)msg);
    ibmp_msg_freelist_verify(env, __LINE__);
    if (! locked) {
	ibmp_unlock(env);
    }

    msg->copy = (int)(*env)->GetBooleanField(env, this, fld_makeCopy);
    IBP_VPRINTF(820, env,
		("Make %s intermediate copy in this BytOutputStream\n",
		 msg->copy ? "an" : "NO"));

    return msg;
}


static void
check_home_msg(jint cpu)
{
    static int first_pass = 1;

    if (cpu == ibmp_me) {
	if (first_pass) {
	    first_pass = 0;
	    fprintf(stderr, "Send message to my own Ibis; implement shortcut?\n");
	}
    }
}


static int
empty_frag(JNIEnv *env,
	   ibmp_msg_p msg,
	   int forceFirstFrag,
	   int lastFrag,
	   int lastSplitter)
{
    if (msg != NULL) {
	if (forceFirstFrag) {
	    msg->firstFrag = 1;
	    send_msg++;
	}

	if ((lastFrag && msg->firstFrag) || msg->iov_len != 0) {
	    if (! lastFrag && lastSplitter) {
		msg->firstFrag = 0;
	    }
	    return 0;
	}
    }

    IBP_VPRINTF(250, env, ("Skip send of an empty non-single fragment msg %p, firstFrag %s lastFrag %s lastSplitter %s\n", msg, forceFirstFrag ? "yes" : "no", lastFrag ? "yes" : "no", lastSplitter ? "yes" : "no"));

    send_frag_skip++;

    return 1;
}


static int
finished_from_send(JNIEnv *env, ibmp_msg_p msg)
{
    /* You never know whether the async msg has already been acked,
     * so let's check.
     * If we would poll the send completion queue here, the sent upcall might
     * be cleared and enqueued and in the wrong state, but a poll of the MP
     * layer below this is OK. */
    ibp_mp_poll(env);

    if (msg->outstanding_send != 0 || msg->state != MSG_STATE_ASYNC_SEND) {
	return 0;
    }

    IBP_VPRINTF(450, env, ("Dequeue msg %p\n", msg));
    ibmp_msg_deq(msg);
    ibmp_msg_release_iov(env, msg);

    return 1;
}


static jboolean
send_async(JNIEnv *env,
	   jobject this,
	   jint cpu,
	   ibmp_msg_p msg,
	   jint i,
	   jint splitTotal)
{
    IBP_VPRINTF(300, env, ("ByteOS %p Enqueue a send-finish upcall msg %p obj %p, missing := %d proto %p\n",
		msg->byte_output_stream, msg, this, ++ibmp_sent_msg_out,
		msg->proto[i]));
#ifdef IBP_VERBOSE
    if (ibmp_verbose >= 1 && ibmp_verbose < 300) {
	++ibmp_sent_msg_out;
    }
#endif

    if (i == 0) {
	assert(msg->state == MSG_STATE_ACCUMULATING);
	assert(msg->byte_output_stream != NULL);
	assert(msg->outstanding_send == 0);

	msg->outstanding_send += splitTotal;
	msg->outstanding_final = 1;

	ibmp_msg_freelist_verify(env, __LINE__);
	assert(msg->next == NULL);

	msg->state = MSG_STATE_ASYNC_SEND;
	ibmp_msg_enq(msg);

	ibmp_msg_freelist_verify(env, __LINE__);
	IBP_VPRINTF(450, env, ("Enqueued msg %p\n", msg));
    } else {
	assert(msg->state == MSG_STATE_ASYNC_SEND);
    }

    ibp_mp_send_async(env, (int)cpu, ibmp_byte_stream_port,
		      msg->iov, msg->iov_len,
		      msg->proto[i], ibmp_byte_stream_proto_size,
		      sent_upcall, msg);

    if (finished_from_send(env, msg)) {
	return JNI_FALSE;
    }

    ibmp_msg_freelist_verify(env, __LINE__);
    IBP_VPRINTF(450, env, ("Msg %p must be acked/released from upcall\n", msg));

    return JNI_TRUE;
}



JNIEXPORT jboolean JNICALL
Java_ibis_impl_messagePassing_ByteOutputStream_msg_1send(
	JNIEnv *env, 
	jobject this,
	jint cpu,
	jint port,
	jint my_port,
	jint msgSeqno,
	jint nativeIOVec,
	jint i,			/* Split count */
	jint splitTotal,
	jboolean forceFirstFrag,
	jboolean lastFrag	/* Frag is sent as last frag of a message */
	)
{
#if JASON
    struct pan_time st;
    struct pan_time et;
#endif
    ibmp_msg_p	msg = (ibmp_msg_p)nativeIOVec;
    ibmp_byte_stream_hdr_p hdr;
    int		len;
    int		up_to_now;
    int		lastSplitter = (i == splitTotal - 1);

#if JASON
    pan_time_get(&st);
#endif

    IBP_VPRINTF(100, env, ("msg_send: %d port %d seqno %d, splitter %d last %d\n", cpu, port, msgSeqno, i, splitTotal));

    if (! ibmp_byte_output_stream_alive) {
	(*env)->ThrowNew(env, cls_java_io_IOException, "Ibis MessagePassing ByteOutputStream closed");
	return JNI_FALSE;
    }

    check_home_msg(cpu);

    send_frag++;
    if (msg != NULL && msg->firstFrag) send_first_frag++;
    if (lastFrag && lastSplitter) send_last_frag++;

    if (empty_frag(env, msg, forceFirstFrag, lastFrag, lastSplitter)) {
	return JNI_FALSE;
    }

    assert(msg != NULL);
    assert(ibmp_equals(env, msg->byte_output_stream, this));

    splitter_increase(msg, splitTotal);

    hdr = ibmp_byte_stream_hdr(msg->proto[i]);

    hdr->dest_port = port;
    hdr->src_port  = my_port;
    if (lastFrag) {
	hdr->msgSeqno = -msgSeqno;
    } else {
	hdr->msgSeqno = msgSeqno;
    }
    hdr->group = ibis_impl_messagePassing_SendPort_NO_BCAST_GROUP;

    len = ibmp_iovec_len(msg->iov, msg->iov_len);
    up_to_now = (*env)->GetIntField(env, this, fld_msgCount);
    (*env)->SetIntField(env, this, fld_msgCount, len + up_to_now);

#ifdef IBP_VERBOSE
    sent_data += len;
#endif

    assert(msg == NULL || msg->byte_output_stream != NULL);

    if (pan_thread_nonblocking() || len >= ibmp_send_sync) {
	IBP_VPRINTF(250, env, ("ByteOS %p Do this send in Async fashion msg %p seqno %d; lastFrag=%s data size %d iov_size %d\n", this, msg, msgSeqno, lastFrag ? "yes" : "no", ibmp_iovec_len(msg->iov, msg->iov_len), msg->iov_len));

	return send_async(env, this, cpu, msg, i, splitTotal);
    }

    /* Wow, we're allowed to do a sync send! */
    IBP_VPRINTF(250, env, ("ByteOS %p Do this send in sync fashion msg %p seqno %d; lastFrag=%s data size %d iov_size %d\n", this, msg, msgSeqno, lastFrag ? "yes" : "no", ibmp_iovec_len(msg->iov, msg->iov_len), msg->iov_len));

    if (i == 0) {
	assert(msg->state == MSG_STATE_ACCUMULATING);
	msg->state = MSG_STATE_SYNC_SEND;
    } else {
	assert(msg->state == MSG_STATE_SYNC_SEND);
    }

    ibp_mp_send_sync(env, (int)cpu, ibmp_byte_stream_port,
		     msg->iov, msg->iov_len,
		     msg->proto[i], ibmp_byte_stream_proto_size);

    if (i == splitTotal - 1) {
	ibmp_msg_release_iov(env, msg);
    }

    return JNI_FALSE;
}


JNIEXPORT jboolean JNICALL
Java_ibis_impl_messagePassing_ByteOutputStream_msg_1bcast(
	JNIEnv *env, 
	jobject this,
	jint group,
	jint msgSeqno,
	jint nativeIOVec,
	jboolean forceFirstFrag,
	jboolean lastFrag	/* Frag is sent as last frag of a message */
	)
{
#if JASON
    struct pan_time st;
    struct pan_time et;
#endif
    ibmp_msg_p	msg = (ibmp_msg_p)nativeIOVec;
    ibmp_byte_stream_hdr_p hdr;
    int		len;
    int		up_to_now;

#if JASON
    pan_time_get(&st);
#endif

    IBP_VPRINTF(100, env, ("msg_bcast: msg %p group %d seqno %d\n", msg, (int)group, (int)msgSeqno));

    if (! ibmp_byte_output_stream_alive) {
	(*env)->ThrowNew(env, cls_java_io_IOException, "Ibis MessagePassing ByteOutputStream closed");
	return JNI_FALSE;
    }

    send_frag++;
    if (msg != NULL && msg->firstFrag) send_first_frag++;
    if (lastFrag) send_last_frag++;

    if (empty_frag(env, msg, forceFirstFrag, lastFrag, JNI_TRUE)) {
	return JNI_FALSE;
    }

    assert(msg != NULL);
    assert(ibmp_equals(env, msg->byte_output_stream, this));

    hdr = ibmp_byte_stream_hdr(msg->proto[0]);

    if (lastFrag) {
	hdr->msgSeqno = -msgSeqno;
    } else {
	hdr->msgSeqno = msgSeqno;
    }
    hdr->group = group;
    hdr->home_msg = msg;

    len = ibmp_iovec_len(msg->iov, msg->iov_len);
    up_to_now = (*env)->GetIntField(env, this, fld_msgCount);
    (*env)->SetIntField(env, this, fld_msgCount, len + up_to_now);

#ifdef IBP_VERBOSE
    sent_data += len;
#endif

    IBP_VPRINTF(250, env, ("ByteOS %p Do this bcast in Async fashion msg %p seqno %d; lastFrag=%s data size %d iov_size %d\n", this, msg, msgSeqno, lastFrag ? "yes" : "no", ibmp_iovec_len(msg->iov, msg->iov_len), msg->iov_len));

    IBP_VPRINTF(300, env, ("ByteOS %p Enqueue a bcast-finish upcall msg %p obj %p, missing := %d proto %p\n",
		msg->byte_output_stream, msg, this, ++ibmp_sent_msg_out,
		msg->proto[0]));
#ifdef IBP_VERBOSE
    if (ibmp_verbose >= 1 && ibmp_verbose < 300) {
	++ibmp_sent_msg_out;
    }
#endif

    assert(msg->state == MSG_STATE_ACCUMULATING);
    assert(msg->byte_output_stream != NULL);
    assert(ibmp_equals(env, msg->byte_output_stream, this));
    assert(msg->outstanding_send == 0);

    msg->outstanding_send++;
    msg->outstanding_final = lastFrag;

    ibmp_msg_freelist_verify(env, __LINE__);
    assert(msg->next == NULL);

    msg->state = MSG_STATE_ASYNC_SEND;
    ibmp_msg_enq(msg);

    ibmp_msg_freelist_verify(env, __LINE__);
    IBP_VPRINTF(450, env, ("Enqueued msg %p object %p\n", msg, msg->byte_output_stream));

    ibp_mp_bcast(env, ibmp_byte_stream_port,
		 msg->iov, msg->iov_len,
		 msg->proto[0], ibmp_byte_stream_proto_size);

    assert(ibmp_equals(env, msg->byte_output_stream, this));

    if (finished_from_send(env, msg)) {
	return JNI_FALSE;
    }

    ibmp_msg_freelist_verify(env, __LINE__);
    IBP_VPRINTF(450, env, ("Msg %p must be acked/released from upcall\n", msg));

    return JNI_TRUE;
}


void
ibmp_bcast_home_ack(ibmp_byte_stream_hdr_p hdr)
{
    ibmp_msg_p msg = hdr->home_msg;

    msg->outstanding_send--;
    IBP_VPRINTF(1, NULL, ("bcast sent upcall msg %p group %d outstanding := %d, missing := %d\n", msg, hdr->group, msg->outstanding_send, --ibmp_sent_msg_out));
}


JNIEXPORT void JNICALL
Java_ibis_impl_messagePassing_ByteOutputStream_init(
	JNIEnv *env,
	jobject this)
{
    ibmp_msg_p msg = ibmp_msg_check(env, this, 0, 0);

    assert(ibmp_equals(env, msg->byte_output_stream, this));
}


JNIEXPORT void JNICALL
Java_ibis_impl_messagePassing_ByteOutputStream_close(
	JNIEnv *env, 
	jobject this)
{
    ibmp_error(env, "close not implemented\n");
}


static void
iovec_grow(JNIEnv *env, ibmp_msg_p msg, int locked)
{
    if (msg->iov_len == msg->iov_alloc_len) {
	if (! locked) {
	    ibmp_lock(env);
	}
	if (msg->iov_alloc_len == 0) {
	    msg->iov_alloc_len = IOV_CHUNK;
	} else {
	    msg->iov_alloc_len *= 2;
	}
	msg->iov = pan_realloc(msg->iov,
			       msg->iov_alloc_len * sizeof(pan_iovec_t));
	msg->release = pan_realloc(msg->release,
				   msg->iov_alloc_len * sizeof(release_t));
	if (! locked) {
	    ibmp_unlock(env);
	}
    }
}


static int
buf_grow(JNIEnv *env, ibmp_msg_p msg, int incr, int locked)
{
    assert(((incr + 7) & ~7) >= incr);

    incr = (incr + 7) & ~7;
    if (msg->buf_len + incr > msg->buf_alloc_len) {
	char *old_buf = msg->buf;
	int i;

	if (! locked) {
	    ibmp_lock(env);
	}
	while (msg->buf_len + incr > msg->buf_alloc_len) {
	    if (msg->buf_alloc_len == 0) {
		msg->buf_alloc_len = BUF_CHUNK;
	    } else {
		msg->buf_alloc_len *= 2;
	    }
	}
	msg->buf = pan_realloc(msg->buf, msg->buf_alloc_len);
	if (! locked) {
	    ibmp_unlock(env);
	}
	for (i = 0; i < msg->iov_len; i++) {
	    if ((char *)(msg->iov[i].data) >= old_buf &&
		(char *)(msg->iov[i].data) < old_buf + msg->buf_len) {
		int diff = (char *) (msg->iov[i].data) - old_buf;
		msg->iov[i].data = msg->buf + diff;
	    }
	}
    }
    return incr;
}


JNIEXPORT void JNICALL
Java_ibis_impl_messagePassing_ByteOutputStream_write(
	JNIEnv *env, 
	jobject this,
	jint b)
{
    jint	m = (*env)->GetIntField(env, this, fld_nativeIOVec);
    ibmp_msg_p	msg = ibmp_msg_check(env, this, m, 0);

    if (! msg->copy) {
	int incr = buf_grow(env, msg, sizeof(jbyte), 0);
	iovec_grow(env, msg, 0);
	*((unsigned char *) (&msg->buf[msg->buf_len])) = (unsigned char)(b & 0xFF);
	msg->iov[msg->iov_len].data = &(msg->buf[msg->buf_len]);
	msg->iov[msg->iov_len].len = sizeof(jbyte);
	msg->release[msg->iov_len].type = jprim_n_types;
	msg->buf_len += incr;
    }
    else {
	unsigned char *buf;

	ibmp_lock(env);
	iovec_grow(env, msg, 1);

	buf = pan_malloc(sizeof(unsigned char));
	ibmp_unlock(env);
	*buf = (unsigned char)(b & 0xFF);
	msg->iov[msg->iov_len].data = buf;
	msg->iov[msg->iov_len].len = sizeof(jbyte);
    }
    IBP_VPRINTF(300, env, ("Now push byte ByteOS %p msg %p data %p size %d iov %d, value %d\n",
		this, msg, msg->iov[msg->iov_len].data, msg->iov[msg->iov_len].len,
		msg->iov_len, b));
    msg->iov_len++;
}


JNIEXPORT void JNICALL
Java_ibis_impl_messagePassing_ByteOutputStream_report(
	JNIEnv *env, 
	jobject this)
{
    jint	m = (*env)->GetIntField(env, this, fld_nativeIOVec);
    ibmp_msg_p	msg = ibmp_msg_check(env, this, m, 0);
    int		total;
    int		i;

    total = 0;
    for (i = 0; i < msg->iov_len; i++) {
	total += msg->iov[i].len;
    }
    IBP_VPRINTF(300, env, ("PandaByteOutputStream: bytes %d items %d\n",
		total, msg->iov_len));
}



#define ARRAY_WRITE(JType, jtype) \
JNIEXPORT void JNICALL \
Java_ibis_impl_messagePassing_ByteOutputStream_write ## JType ## Array( \
	JNIEnv *env,  \
	jobject this, \
	jtype ## Array b, \
	jint off, \
	jint len) \
{ \
    jint	m = (*env)->GetIntField(env, this, fld_nativeIOVec); \
    ibmp_msg_p	msg = ibmp_msg_check(env, this, m, 0); \
    jtype      *buf; \
    int		sz = len * sizeof(jtype); \
    \
    if (! msg->copy && sz >= COPY_THRESHOLD) { \
	b = (*env)->NewGlobalRef(env, b); \
	IBP_VPRINTF(800, env, ("%s: Now create global ref %p\n", \
		    ibmp_currentThread(env), b)); \
    } \
    if (msg->copy) { \
	ibmp_lock(env); \
	iovec_grow(env, msg, 1); \
	buf = pan_malloc(sz); \
	ibmp_unlock(env); \
	msg->iov[msg->iov_len].data = buf; \
	msg->iov[msg->iov_len].len = sz; \
	(*env)->Get ## JType ## ArrayRegion(env, b, (jsize) off, (jsize) len, (jtype *) buf); \
    } else { \
	iovec_grow(env, msg, 0); \
	if (sz < COPY_THRESHOLD) { \
	    int incr = buf_grow(env, msg, sz, 0); \
	    (*env)->Get ## JType ## ArrayRegion(env, b, (jsize) off, (jsize) len, (jtype *) &(msg->buf[msg->buf_len])); \
	    msg->iov[msg->iov_len].data = &(msg->buf[msg->buf_len]); \
	    msg->buf_len += incr; \
	    msg->iov[msg->iov_len].len  = sz; \
	    msg->release[msg->iov_len].type  = jprim_n_types; \
	} \
	else { \
	    jtype *a = (*env)->Get ## JType ## ArrayElements(env, b, NULL); \
	    msg->iov[msg->iov_len].data = a + off; \
	    msg->iov[msg->iov_len].len  = sz; \
	    msg->release[msg->iov_len].array = b; \
	    msg->release[msg->iov_len].buf   = a; \
	    msg->release[msg->iov_len].type  = jprim_ ## JType; \
	} \
    } \
    IBP_VPRINTF(300, env, ("Now push ByteOS %p msg %p %s source %p data %p size %d iov %d total %d [%d,%d,%d,%d,...]\n", \
		this, msg, #JType, b, msg->iov[msg->iov_len].data, \
		msg->iov[msg->iov_len].len, msg->iov_len, ibmp_iovec_len(msg->iov, msg->iov_len + 1), msg->iov[0].len, msg->iov[1].len, msg->iov[2].len, msg->iov[3].len)); \
    dump_ ## jtype((jtype *)(msg->iov[msg->iov_len].data), len); \
    msg->iov_len++; \
}

ARRAY_WRITE(Boolean, jboolean)
ARRAY_WRITE(Byte, jbyte)
ARRAY_WRITE(Char, jchar)
ARRAY_WRITE(Short, jshort)
ARRAY_WRITE(Int, jint)
ARRAY_WRITE(Long, jlong)
ARRAY_WRITE(Float, jfloat)
ARRAY_WRITE(Double, jdouble)


void
ibmp_byte_output_stream_report(JNIEnv *env)
{
#ifdef IBP_VERBOSE
    fprintf(stderr, "%2d: PandaBufferedOutputStream.sent data %d\n", ibmp_me, sent_data);
#endif
#if DISABLE_SENDER_INTERRUPTS
    fprintf(stderr, "%2d: IBP intr enable %d disable %d send msg %d frag %d (first %d last %d skip %d) \n",
	    ibmp_me, intr_enable, intr_disable, send_msg, send_frag, send_first_frag, send_last_frag, send_frag_skip);
#else
    fprintf(stderr, "%2d: IBP send msg %d frag %d (skip %d) \n",
	    ibmp_me, send_msg, send_frag, send_frag_skip);
#endif
}


void
ibmp_byte_output_stream_init(JNIEnv *env)
{
    cls_PandaByteOutputStream = (*env)->FindClass(env,
			 "ibis/impl/messagePassing/ByteOutputStream");
    if (cls_PandaByteOutputStream == NULL) {
	ibmp_error(env, "Cannot find class ibis/impl/messagePassing/ByteOutputStream\n");
    }
    cls_PandaByteOutputStream = (jclass)(*env)->NewGlobalRef(env, (jobject)cls_PandaByteOutputStream);

    fld_nativeIOVec        = (*env)->GetFieldID(env,
					 cls_PandaByteOutputStream,
					 "nativeIOVec", "I");
    if (fld_nativeIOVec == NULL) {
	ibmp_error(env, "Cannot find static field nativeIOVec:I\n");
    }

    fld_waitingInPoll    = (*env)->GetFieldID(env,
					 cls_PandaByteOutputStream,
					 "waitingInPoll", "Z");
    if (fld_waitingInPoll == NULL) {
	ibmp_error(env, "Cannot find static field waitingInPoll:Z\n");
    }

    fld_outstandingFrags = (*env)->GetFieldID(env,
					 cls_PandaByteOutputStream,
					 "outstandingFrags", "I");
    if (fld_outstandingFrags == NULL) {
	ibmp_error(env, "Cannot find static field outstandingFrags:I\n");
    }

    fld_makeCopy = (*env)->GetFieldID(env,
					 cls_PandaByteOutputStream,
					 "makeCopy", "Z");
    if (fld_makeCopy == NULL) {
	ibmp_error(env, "Cannot find static field makeCopy:Z\n");
    }

    fld_msgCount = (*env)->GetFieldID(env,
					 cls_PandaByteOutputStream,
					 "msgCount", "I");
    if (fld_msgCount == NULL) {
	ibmp_error(env, "Cannot find static field msgCount:I\n");
    }

    md_finished_upcall    = (*env)->GetMethodID(env,
						cls_PandaByteOutputStream,
						"finished_upcall", "()V");
    if (md_finished_upcall == NULL) {
	ibmp_error(env, "Cannot find method finished_upcall()V\n");
    }

    ibmp_byte_stream_port = ibp_mp_port_register(ibmp_byte_stream_handle);
    ibmp_byte_stream_proto_start = align_to(ibp_mp_proto_offset(), ibmp_byte_stream_hdr_t);
    ibmp_byte_stream_proto_size  = ibmp_byte_stream_proto_start + sizeof(ibmp_byte_stream_hdr_t);

    ibmp_poll_register(ibmp_msg_q_poll);

    if (pan_arg_int(NULL, NULL, "-ibp-send-sync", &ibmp_send_sync) == -1) {
	ibmp_error(env, "-ibp-send-sync requires an int argument\n");
    }

    ibmp_byte_output_stream_alive = 1;
#if JASON
    pan_time_get(&start);
#endif
}


void
ibmp_byte_output_stream_end(JNIEnv *env)
{
    ibmp_byte_output_stream_alive = 0;

    ibmp_byte_output_stream_report(env);
}
